/*************************************************************************
 *
 *  $RCSfile: DateTimeHelper.cxx,v $
 *
 *  $Revision: 1.4 $
 *
 *  last change: $Author: kso $ $Date: 2001-06-21 10:13:54 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <stdio.h>

#ifndef _OSL_TIME_H_
#include <osl/time.h>
#endif

#ifndef _COM_SUN_STAR_UTIL_DATETIME_HPP_
#include <com/sun/star/util/DateTime.hpp>
#endif

#ifndef _WEBDAV_DATETIME_HELPER_HXX
#include "DateTimeHelper.hxx"
#endif

using namespace com::sun::star::util;
using namespace rtl;

using namespace webdav_ucp;

bool DateTimeHelper::ISO8601_To_DateTime (const OUString& s,
    DateTime& dateTime)
{
    OString aDT (s.getStr(), s.getLength(), RTL_TEXTENCODING_ASCII_US);

    int year, month, day, hours, minutes, off_hours, off_minutes, fix;
    double seconds;

    // 2001-01-01T12:30:00Z
    int n = sscanf( aDT.getStr(), "%04d-%02d-%02dT%02d:%02d:%lfZ",
                    &year, &month, &day, &hours, &minutes, &seconds );
    if ( n == 6 )
    {
        fix = 0;
    }
    else
    {
        // 2001-01-01T12:30:00+03:30
        n = sscanf( aDT.getStr(), "%04d-%02d-%02dT%02d:%02d:%lf+%02d:%02d",
                    &year, &month, &day, &hours, &minutes, &seconds,
                    &off_hours, &off_minutes );
        if ( n == 8 )
        {
            fix = - off_hours * 3600 - off_minutes * 60;
        }
        else
        {
            // 2001-01-01T12:30:00-03:30
            n = sscanf( aDT.getStr(), "%04d-%02d-%02dT%02d:%02d:%lf-%02d:%02d",
                        &year, &month, &day, &hours, &minutes, &seconds,
                        &off_hours, &off_minutes );
            if ( n = 8 )
            {
                fix = off_hours * 3600 + off_minutes * 60;
            }
            else
            {
                return false;
            }
        }
    }

    // Convert to local time...

    oslDateTime aDateTime;
    aDateTime.NanoSeconds = 0;
    aDateTime.Seconds     = static_cast< int >( seconds );    // 0-59
    aDateTime.Minutes     = minutes;    // 0-59
    aDateTime.Hours       = hours;      // 0-23
    aDateTime.Day         = day;        // 1-31
    aDateTime.DayOfWeek   = 0;          // 0-6, 0 = Sunday
    aDateTime.Month       = month;      // 1-12
    aDateTime.Year        = year;

    TimeValue aTimeValue;
    if ( osl_getTimeValueFromDateTime( &aDateTime, &aTimeValue ) )
    {
        aTimeValue.Seconds += fix;

        if ( osl_getLocalTimeFromSystemTime( &aTimeValue, &aTimeValue ) )
        {
            if ( osl_getDateTimeFromTimeValue( &aTimeValue, &aDateTime ) )
            {
                dateTime.Year    = aDateTime.Year;
                dateTime.Month   = aDateTime.Month;
                dateTime.Day     = aDateTime.Day;
                dateTime.Hours   = aDateTime.Hours;
                dateTime.Minutes = aDateTime.Minutes;
                dateTime.Seconds = aDateTime.Seconds;

                return true;
             }
        }
    }

    return false;
}

/*
sal_Int32 DateTimeHelper::convertDayToInt (const OUString& day)
{
    if (day.compareToAscii ("Sun") == 0)
        return 0;
    else if (day.compareToAscii ("Mon") == 0)
        return 1;
    else if (day.compareToAscii ("Tue") == 0)
        return 2;
    else if (day.compareToAscii ("Wed") == 0)
        return 3;
    else if (day.compareToAscii ("Thu") == 0)
        return 4;
    else if (day.compareToAscii ("Fri") == 0)
        return 5;
    else if (day.compareToAscii ("Sat") == 0)
        return 6;
    else
        return -1;
}
*/

sal_Int32 DateTimeHelper::convertMonthToInt (const OUString& month)
{
    if (month.compareToAscii ("Jan") == 0)
        return 1;
    else if (month.compareToAscii ("Feb") == 0)
        return 2;
    else if (month.compareToAscii ("Mar") == 0)
        return 3;
    else if (month.compareToAscii ("Apr") == 0)
        return 4;
    else if (month.compareToAscii ("May") == 0)
        return 5;
    else if (month.compareToAscii ("Jun") == 0)
        return 6;
    else if (month.compareToAscii ("Jul") == 0)
        return 7;
    else if (month.compareToAscii ("Aug") == 0)
        return 8;
    else if (month.compareToAscii ("Sep") == 0)
        return 9;
    else if (month.compareToAscii ("Oct") == 0)
        return 10;
    else if (month.compareToAscii ("Nov") == 0)
        return 11;
    else if (month.compareToAscii ("Dec") == 0)
        return 12;
    else
        return 0;
}

bool DateTimeHelper::RFC2068_To_DateTime (const OUString& s,
    DateTime& dateTime)
{
    int year;
    int day;
    int hours;
    int minutes;
    int seconds;
    sal_Char string_month[3 + 1];
    sal_Char string_day[3 + 1];

    sal_Int32 found = s.indexOf (',');
    if (found != -1)
    {
        OString aDT (s.getStr(), s.getLength(), RTL_TEXTENCODING_ASCII_US);

        // RFC 1123
        found = sscanf (aDT.getStr(), "%3s, %2d %3s %4d %2d:%2d:%2d GMT",
                        string_day, &day, string_month, &year, &hours, &minutes, &seconds);
        if (found != 7)
        {
            // RFC 1036
            found = sscanf (aDT.getStr(), "%3s, %2d-%3s-%2d %2d:%2d:%2d GMT",
                            string_day, &day, &string_month, &year, &hours, &minutes, &seconds);
        }
        found = (found == 7) ? 1 : 0;
    }
    else
    {
        OString aDT (s.getStr(), s.getLength(), RTL_TEXTENCODING_ASCII_US);

        // ANSI C's asctime () format
        found = sscanf (aDT.getStr(), "%3s %3s %d %2d:%2d:%2d %4d",
                        string_day, string_month,
                        &day, &hours, &minutes, &seconds, &year);
        found = (found == 7) ? 1 : 0;
    }

    if (found)
    {
        found = 0;

        int month = DateTimeHelper::convertMonthToInt (
                            OUString::createFromAscii (string_month));
        if (month)
        {
            // Convert to local time...

            oslDateTime aDateTime;
            aDateTime.NanoSeconds = 0;
            aDateTime.Seconds     = seconds;    // 0-59
            aDateTime.Minutes     = minutes;    // 0-59
            aDateTime.Hours       = hours;      // 0-23
            aDateTime.Day         = day;        // 1-31
            aDateTime.DayOfWeek   = 0; //dayofweek;  // 0-6, 0 = Sunday
            aDateTime.Month       = month;      // 1-12
            aDateTime.Year        = year;

            TimeValue aTimeValue;
            if ( osl_getTimeValueFromDateTime( &aDateTime,
                                                &aTimeValue ) )
            {
                if ( osl_getLocalTimeFromSystemTime( &aTimeValue,
                                                        &aTimeValue ) )
                {
                    if ( osl_getDateTimeFromTimeValue( &aTimeValue,
                                                        &aDateTime ) )
                    {
                        dateTime.Year    = aDateTime.Year;
                        dateTime.Month   = aDateTime.Month;
                        dateTime.Day     = aDateTime.Day;
                        dateTime.Hours   = aDateTime.Hours;
                        dateTime.Minutes = aDateTime.Minutes;
                        dateTime.Seconds = aDateTime.Seconds;

                        found = 1;
                    }
                }
            }
        }
    }

    return (found) ? true : false;
}

bool DateTimeHelper::convert (const OUString& s, DateTime& dateTime)
{
    if (ISO8601_To_DateTime (s, dateTime))
        return true;
    else if (RFC2068_To_DateTime (s, dateTime))
        return true;
    else
        return false;
}

